@@ -1,6 +1,6 @@
 package Cassandra::Client::AsyncEV;
 our $AUTHORITY = 'cpan:TVDW';
-$Cassandra::Client::AsyncEV::VERSION = '0.13';
+$Cassandra::Client::AsyncEV::VERSION = '0.14';
 use 5.010;
 use strict;
 use warnings;
@@ -130,6 +130,11 @@ sub timer {
     });
 }
 
+sub later {
+    my ($self, $callback)= @_;
+    $self->timer($callback, 0);
+}
+
 # $something->($async->wait(my $w)); my ($error, $result)= $w->();
 sub wait {
     my ($self)= @_;
@@ -169,7 +174,7 @@ Cassandra::Client::AsyncEV
 
 =head1 VERSION
 
-version 0.13
+version 0.14
 
 =head1 AUTHOR
 
