@@ -1,8 +1,7 @@
 package Cassandra::Client::Error::WriteTimeoutException;
 our $AUTHORITY = 'cpan:TVDW';
-$Cassandra::Client::Error::WriteTimeoutException::VERSION = '0.14_003'; # TRIAL
-
-$Cassandra::Client::Error::WriteTimeoutException::VERSION = '0.14003';use parent 'Cassandra::Client::Error::Base';
+$Cassandra::Client::Error::WriteTimeoutException::VERSION = '0.14';
+use parent 'Cassandra::Client::Error::Base';
 use 5.010;
 use strict;
 use warnings;
@@ -25,7 +24,7 @@ Cassandra::Client::Error::WriteTimeoutException
 
 =head1 VERSION
 
-version 0.14_003
+version 0.14
 
 =head1 AUTHOR
 
@@ -33,7 +32,7 @@ Tom van der Woerdt <tvdw@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2019 by Tom van der Woerdt.
+This software is copyright (c) 2017 by Tom van der Woerdt.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
